
from django.db.models import Manager
from products.querysets import ProductQuerySet


class ProductManager(Manager):

    def get_queryset(self):
        return ProductQuerySet(self.model, using=self._db)

    def visible(self):
        return self.get_queryset().visible()

    def for_category(self, category):
        return self.get_queryset().for_category(category)

    def search(self, **kwargs):
        return self.get_queryset().search(**kwargs)

    def with_attr_values(self, value_ids):
        return self.get_queryset().with_attr_values(value_ids)




from django.db import models
from django.utils.translation import ugettext_lazy as _


class IsVisibleField(models.BooleanField):

    def __init__(
            self,
            verbose_name=_('Is visible'),
            default=True,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            default=default,
            *args, **kwargs)


class IsNewField(models.BooleanField):

    def __init__(
            self,
            verbose_name=_('Is new'),
            default=False,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            default=default,
            *args, **kwargs)


class NameField(models.CharField):

    def __init__(
            self,
            verbose_name=_('Product name'),
            max_length=255,
            blank=True,
            db_index=True,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            max_length=max_length,
            blank=blank,
            db_index=db_index,
            *args, **kwargs)


class CodeField(models.CharField):

    def __init__(
            self,
            verbose_name=_('Code'),
            max_length=255,
            blank=True,
            db_index=True,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            max_length=max_length,
            blank=blank,
            db_index=db_index,
            *args, **kwargs)


class AdditionalCodesField(models.CharField):

    def __init__(
            self,
            verbose_name=_('Additional codes'),
            max_length=2700,
            blank=True,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            max_length=max_length,
            blank=blank,
            *args, **kwargs)


class PriceField(models.FloatField):

    def __init__(
            self,
            verbose_name=_('Price'),
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            *args, **kwargs)


class OldPriceField(models.FloatField):

    def __init__(
            self,
            verbose_name=_('Old price'),
            blank=True,
            null=True,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            blank=blank,
            null=null,
            *args, **kwargs)


class DescriptionField(models.TextField):

    def __init__(
            self,
            verbose_name=_('Description'),
            blank=True,
            *args, **kwargs):
        super().__init__(
            verbose_name=verbose_name,
            blank=blank,
            *args, **kwargs)




from random import randint


def add_product_to_history(request, product_id):

    product_ids = request.session.get('PRODUCT_HISTORY', [])

    if product_id in product_ids:
        product_ids.remove(product_id)

    product_ids.insert(0, product_id)

    request.session['PRODUCT_HISTORY'] = product_ids


def get_products_from_history(request, queryset, count=6):

    ids = request.session.get('PRODUCT_HISTORY', [])[:count]

    if not ids:
        return []

    return queryset.filter(id__in=ids)


def get_related_products(queryset, product_id, count=6):

    index = 0

    related_products = queryset.exclude(pk=product_id)

    related_products_count = len(related_products)

    if related_products_count:

        if related_products_count > count:
            index = randint(0, related_products_count - count)

        return related_products[index:index + count]

    return []




from django.db.models import Q, QuerySet

from modeltranslation.utils import get_translation_fields
from model_search import model_search


class ProductQuerySet(QuerySet):

    def visible(self):
        return self.filter(is_visible=True)

    def for_category(self, category):
        return self.filter(category=category)

    def search(self, query=None):

        queryset = self

        if query:
            queryset = model_search(query, queryset, (
                ['code'] +
                get_translation_fields('name') +
                get_translation_fields('description')
            ))

        return queryset

    def with_attr_values(self, value_ids):

        if value_ids:
            return self.filter(attr_values__value_option__in=value_ids)

        return self
